/****************************************************************************
* gi-joe : Gentoo's installer project
*
* detect.c : python bindings for libdetect
*
* Author: Meir Kriheli <meir@mksoft.co.il>
* License : GPL-2
* 
*****************************************************************************/

#include <Python.h>
#include <detect.h>

// defince intital structure pointer
struct cards_lst *lst;
struct bus_lst *bus;
struct cpu_info *cpu;
struct ethernet_info *ether;
struct tvcard_info *tvcard;
struct soundcard_info *soundcard;
struct video_info *video;
struct ide_info *ide;
struct scsi_info *scsi;
struct parallel_info *parallel;
struct serial_info *serial;
struct memory_info *memory;
struct modem_info *modem;
struct scsintf_info *scsintf;
struct cdrom_info *cdrom;
struct floppy_info *floppy;
struct isdn_info *isdn;
struct printer_info *printer;
struct scanner_info *scanner;
struct mouse_info *mouse;
struct tape_info *tape;
struct others_info *others;
struct usbntf_info *usbntf;
struct usb_info *usb;
struct joystick_info *joystick;
struct gameport_info *gameport;

static PyObject *gi_detect_init_bus(PyObject *self,PyObject *args) {
	char *isa_lst, *pci_lst, *pcmcia_lst, *usb_lst;
	
	if (!PyArg_ParseTuple(args, "ssss", &isa_lst, &pci_lst, &pcmcia_lst, &usb_lst ))
		return NULL;
	
	// if we have 4 args for file paths, lets init it, and return nothing
	lst = init_lst(isa_lst, pci_lst, pcmcia_lst, usb_lst);
	bus = init_bus(lst);
	
	Py_INCREF(Py_None);
	return Py_None;
}

void set_dict_string(PyObject *dict, char *key, char *value) {
	PyObject *PyStrKey, *PyStrValue;
	
	PyStrKey = PyString_FromString(key);
	
	// verify that value is a valid pointer (not null)
	// we don't want segmentation faults
	if (value) {
		PyStrValue = PyString_FromString(value);
	}
	else {
		PyStrValue = PyString_FromString("");
	}
	
	PyDict_SetItem(dict,PyStrKey, PyStrValue);
	
	Py_DECREF(PyStrKey);
	Py_DECREF(PyStrValue);
}

void set_dict_int(PyObject *dict, char *key, long value) {
	PyObject *PyStrKey, *PyIntValue;
	
	PyStrKey = PyString_FromString(key);
	PyIntValue = PyInt_FromLong(value);
	
	PyDict_SetItem(dict,PyStrKey, PyIntValue);
	
	Py_DECREF(PyStrKey);
	Py_DECREF(PyIntValue);
}

static PyObject *gi_detect_cpu() {
	
	PyObject *dict, *list;
	PyObject *key, *value;

	list = PyList_New(0);
	// loop through cpus and create a list of dictionaries
	for (cpu = cpu_detect(); cpu; cpu = cpu->next) {
		dict = PyDict_New();
	
		// create the cpu_info dictinary
		set_dict_int(dict, "proc_id", cpu->proc_id);
		set_dict_string(dict, "vendor", cpu->vendor);
		set_dict_string(dict, "model", cpu->model);
		set_dict_int(dict, "freq", cpu->freq);
		set_dict_string(dict, "cache", cpu->cache);
		set_dict_string(dict, "bogomips", cpu->bogomips);
		set_dict_string(dict, "bugs", cpu->bugs);
		set_dict_int(dict, "options", cpu->options);
		// add the dict to the cpu list
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_ethernet() {
	PyObject *dict, *list;
	PyObject *key, *value;

	list = PyList_New(0);

	for (ether = ethernet_detect(bus); ether; ether = ether->next) {
		dict = PyDict_New();

		set_dict_string(dict, "vendor", ether->vendor);
		set_dict_string(dict, "model", ether->model);
		set_dict_string(dict, "module", ether->module);
		set_dict_int(dict, "bus", ether->bus);
		set_dict_int(dict, "io", ether->io);
		set_dict_int(dict, "irq", ether->irq);
 
		// create the ethernet_info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_tvcard() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);

	for (tvcard = tvcard_detect(bus); tvcard; tvcard = tvcard->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", tvcard->vendor);
		set_dict_string(dict, "model", tvcard->model);
		set_dict_string(dict, "module", tvcard->module);
		set_dict_int(dict, "bus", tvcard->bus);
		
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}


static PyObject *gi_detect_soundcard() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);

	for (soundcard = soundcard_detect(bus); soundcard; soundcard = soundcard->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", soundcard->vendor);
		set_dict_string(dict, "model", soundcard->model);
		set_dict_string(dict, "module", soundcard->module);
		set_dict_int(dict, "bus", soundcard->bus);
		set_dict_int(dict, "io", soundcard->io);
		set_dict_int(dict, "irq", soundcard->irq);
		set_dict_int(dict, "dma8", soundcard->dma8);
		set_dict_int(dict, "dma16", soundcard->dma16);
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_video() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);

	for (video = video_detect(bus); video; video = video->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", video->vendor);
		set_dict_string(dict, "model", video->model);
		set_dict_string(dict, "server", video->server);
		set_dict_int(dict, "bus", video->bus);
		set_dict_int(dict, "memory", video->memory);
		set_dict_string(dict, "ramdac", video->ramdac);
		set_dict_string(dict, "clockchip", video->clockchip);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_ide() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	// loop through cpus and create a list of dictionaries
	for (ide = bus->ide; ide; ide = ide->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "device", ide->device);
		set_dict_string(dict, "model", ide->model);
		set_dict_int(dict, "type", ide->type);
		set_dict_int(dict, "cylinders", ide->cylinders);
		set_dict_int(dict, "heads", ide->heads);
		set_dict_int(dict, "sectors", ide->sectors);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_scsi() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (scsi = bus->scsi; scsi; scsi = scsi->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", scsi->vendor);
		set_dict_string(dict, "device", scsi->device);
		set_dict_string(dict, "model", scsi->model);
		set_dict_int(dict, "type", scsi->type);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

// TODO: segfaults. check why
static PyObject *gi_detect_parallel() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (parallel = bus->parallel; parallel; parallel = parallel->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "device", parallel->device);
		set_dict_string(dict, "dos_equivalent", parallel->dos_equivalent);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_serial() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (serial = bus->serial; serial; serial = serial->next) {
		dict = PyDict_New();
	
		set_dict_string(dict, "device", serial->device);
		set_dict_string(dict, "dos_equivalent", serial->dos_equivalent);
		set_dict_string(dict, "vendor", serial->vendor);
		set_dict_string(dict, "model", serial->model);
		set_dict_string(dict, "module", serial->modulename);
		set_dict_int(dict, "speed", serial->speed);
		set_dict_int(dict, "type", serial->type);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_memory() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	if (memory = memory_detect()) {
		dict = PyDict_New();
	
		set_dict_int(dict, "total", memory->total);
		set_dict_int(dict, "free", memory->free);
		set_dict_int(dict, "shared", memory->shared);
		set_dict_int(dict, "buffers", memory->buffers);
		set_dict_int(dict, "cached", memory->cached);
		set_dict_int(dict, "swap_total", memory->swap_total);
		set_dict_int(dict, "swap_free", memory->swap_free);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_modem() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (modem = modem_detect(bus); modem; modem = modem->next ) {
		dict = PyDict_New();
		set_dict_string(dict, "board_id", modem->board_id);
		set_dict_int(dict, "board_num", modem->board_num);
		
		set_dict_string(dict, "dev_id", modem->dev_id);

		set_dict_int(dict, "dev_num", modem->dev_num);
		set_dict_string(dict, "vendor", modem->vendor);
		set_dict_string(dict, "model", modem->model);
		set_dict_string(dict, "device", modem->device);
		set_dict_string(dict, "module", modem->module);
		set_dict_int(dict, "type", modem->bus);
		set_dict_int(dict, "io", modem->io);
		set_dict_int(dict, "irq", modem->irq);
		set_dict_int(dict, "speed", modem->speed);
		set_dict_int(dict, "long_id", modem->long_id);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_scsiInterface() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (scsintf = scsiinterface_detect(bus); scsintf; scsintf = scsintf->next ) {
		dict = PyDict_New();
		
		set_dict_string(dict, "board_id", scsintf->board_id);
		set_dict_int(dict, "board_num", scsintf->board_num);
		set_dict_string(dict, "dev_id", scsintf->dev_id);
		set_dict_int(dict, "dev_num", scsintf->dev_num);
		set_dict_string(dict, "vendor", scsintf->vendor);
		set_dict_string(dict, "model", scsintf->model);
		set_dict_string(dict, "module", scsintf->module);
		set_dict_int(dict, "type", scsintf->bus);
		set_dict_int(dict, "io", scsintf->io);
		set_dict_int(dict, "irq", scsintf->irq);
		set_dict_int(dict, "long_id", scsintf->long_id);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}


static PyObject *gi_detect_cdrom() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (cdrom = cdrom_detect(bus); cdrom; cdrom = cdrom->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", cdrom->vendor);
		set_dict_string(dict, "model", cdrom->model);
		set_dict_string(dict, "module", cdrom->module);
		set_dict_string(dict, "device", cdrom->device);
		set_dict_int(dict, "type", cdrom->bus);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}


static PyObject *gi_detect_floppy() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (floppy = floppy_detect(bus); floppy; floppy = floppy->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", floppy->vendor);
		set_dict_string(dict, "model", floppy->model);
		set_dict_string(dict, "module", floppy->module);
		set_dict_string(dict, "device", floppy->device);
		set_dict_int(dict, "type", floppy->bus);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}


static PyObject *gi_detect_isdn() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (isdn = isdn_detect(bus); isdn; isdn = isdn->next ) {
		dict = PyDict_New();
		
		set_dict_string(dict, "board_id", isdn->board_id);
		set_dict_int(dict, "board_num", isdn->board_num);
		set_dict_string(dict, "dev_id", isdn->dev_id);
		set_dict_int(dict, "dev_num", isdn->dev_num);
		set_dict_string(dict, "vendor", isdn->vendor);
		set_dict_string(dict, "model", isdn->model);
		set_dict_string(dict, "module", isdn->module);
		set_dict_int(dict, "type", isdn->bus);
		set_dict_int(dict, "io", isdn->io);
		set_dict_int(dict, "irq", isdn->irq);
		set_dict_int(dict, "long_id", isdn->long_id);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_printer() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (printer = printer_detect(bus); printer; printer = printer->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", printer->vendor);
		set_dict_string(dict, "model", printer->model);
		set_dict_string(dict, "module", printer->module);
		set_dict_string(dict, "device", printer->device);
		set_dict_int(dict, "type", printer->bus);
		set_dict_int(dict, "long_id", printer->long_id);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}


static PyObject *gi_detect_scanner() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (scanner = scanner_detect(bus); scanner; scanner = scanner->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", scanner->vendor);
		set_dict_string(dict, "model", scanner->model);
		set_dict_string(dict, "module", scanner->module);
		set_dict_string(dict, "device", scanner->device);
		set_dict_int(dict, "type", scanner->bus);
		set_dict_int(dict, "long_id", scanner->long_id);

		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_mouse() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (mouse = mouse_detect(bus); mouse; mouse = mouse->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", mouse->vendor);
		set_dict_string(dict, "model", mouse->model);
		set_dict_string(dict, "module", mouse->module);
		set_dict_string(dict, "device", mouse->device);
		set_dict_int(dict, "type", mouse->bus);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_tape() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (tape = tape_detect(bus); tape; tape = tape->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", tape->vendor);
		set_dict_string(dict, "model", tape->model);
		set_dict_string(dict, "module", tape->module);
		set_dict_string(dict, "device", tape->device);
		set_dict_int(dict, "type", tape->bus);
		
		// create the info dictinary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_others() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (others = others_detect(bus); others; others = others->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", others->vendor);
		set_dict_string(dict, "model", others->model);
		set_dict_string(dict, "module", others->module);
		set_dict_string(dict, "device", others->device);
		set_dict_int(dict, "type", others->bus);
		
		set_dict_string(dict,"board_id", others->board_id);
		set_dict_int(dict, "board_num", others->board_num);
		set_dict_string(dict,"dev_id", others->dev_id);
		set_dict_int(dict, "dev_num", others->dev_num);
		set_dict_int(dict, "io", others->io);
		set_dict_int(dict, "irq", others->irq);
		set_dict_int(dict, "id", others->id);

		// create the info dictionary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_usbInterface() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (usbntf = usbinterface_detect(bus); usbntf; usbntf = usbntf->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", usbntf->vendor);
		set_dict_string(dict, "model", usbntf->model);
		set_dict_string(dict, "module", usbntf->module);
		set_dict_int(dict, "type", usbntf->bus);
		
		set_dict_string(dict,"board_id", usbntf->board_id);
		set_dict_int(dict, "board_num", usbntf->board_num);
		set_dict_string(dict,"dev_id", usbntf->dev_id);
		set_dict_int(dict, "dev_num", usbntf->dev_num);
		set_dict_int(dict, "io", usbntf->io);
		set_dict_int(dict, "irq", usbntf->irq);
		set_dict_int(dict, "long_id", usbntf->long_id);

		// create the info dictionary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_usb() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (usb = usb_detect(lst); usb; usb = usb->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", usb->vendor);
		set_dict_string(dict, "model", usb->model);
		set_dict_string(dict, "module", usb->modulename);
		set_dict_int(dict, "device_type", usb->type);
		set_dict_int(dict, "id", usb->id);

		// create the info dictionary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_joystick() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (joystick = joystick_detect(bus); joystick; joystick = joystick->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", joystick->vendor);
		set_dict_string(dict, "model", joystick->model);
		set_dict_string(dict, "module", joystick->module);
		set_dict_string(dict, "device", joystick->device);
		set_dict_int(dict, "type", joystick->bus);
		// create the info dictionary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyObject *gi_detect_gameport() {
	PyObject *dict, *list;
	PyObject *key, *value;
	
	list = PyList_New(0);
	
	for (gameport = gameport_detect(bus); gameport; gameport = gameport->next ) {
		dict = PyDict_New();
	
		set_dict_string(dict, "vendor", gameport->vendor);
		set_dict_string(dict, "model", gameport->model);
		set_dict_int(dict, "io", gameport->io);
		set_dict_int(dict, "irq", gameport->irq);
		// create the info dictionary
		PyList_Append(list,dict);
		Py_DECREF(dict);
	}
	return list;
}

static PyMethodDef DetectMethods[] = {
	{"init_bus", gi_detect_init_bus, METH_VARARGS, "Initialise bus"},
	{"cpu", gi_detect_cpu, METH_VARARGS, "Detect cpu(s)"},
	{"ethernet", gi_detect_ethernet, METH_VARARGS, "Detect NIC(s)"},
	{"tvcard", gi_detect_tvcard, METH_VARARGS, "Detect TV card(s)"},
	{"soundcard", gi_detect_soundcard, METH_VARARGS, "Detect sound card(s)"},
	{"video", gi_detect_video, METH_VARARGS, "Detect video devices"},
	{"ide", gi_detect_ide, METH_VARARGS, "Detect IDE devices"},
	{"scsi", gi_detect_scsi, METH_VARARGS, "Detect SCSI devices"},
	{"parallel", gi_detect_parallel, METH_VARARGS, "Detect parallel devices"},
	{"serial", gi_detect_serial, METH_VARARGS, "Detect serial devices"},
	{"memory", gi_detect_memory, METH_VARARGS, "Detect memory"},
	{"modem", gi_detect_modem, METH_VARARGS, "Detect modems"},
	{"scsiInterface", gi_detect_scsiInterface, METH_VARARGS, "Detect SCSI controllers"},
	{"cdrom", gi_detect_cdrom, METH_VARARGS, "Detect cdrom(s)"},
	{"floppy", gi_detect_floppy, METH_VARARGS, "Detect floppy(ies)"},
	{"isdn", gi_detect_isdn, METH_VARARGS, "Detect ISDN"},
	{"printer", gi_detect_printer, METH_VARARGS, "Detect printer(s)"},
	{"scanner", gi_detect_scanner, METH_VARARGS, "Detect scanner(s)"},
	{"mouse", gi_detect_mouse, METH_VARARGS, "Detect mouse(s)"},
	{"tape", gi_detect_tape, METH_VARARGS, "Detect tape(s)"},
	{"others", gi_detect_others, METH_VARARGS, "Detect others(s)"},
	{"usbInterface", gi_detect_usbInterface, METH_VARARGS, "Detect USB interface(s)"},
	{"usb", gi_detect_usb, METH_VARARGS, "Detect USB devices(s)"},
	{"joystick", gi_detect_joystick, METH_VARARGS, "Detect joystick(s)"},
	{"gameport", gi_detect_gameport, METH_VARARGS, "Detect gameports(s)"},
	{NULL, NULL, 0, NULL}        /* Sentinel */
};

void initgi_detect(void) {
	(void) Py_InitModule("gi_detect", DetectMethods);
}

